@@ -19,17 +19,17 @@
 #include <apt-pkg/strutl.h>
 
 #include <algorithm>
+#include <cctype>
+#include <cstddef>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <string>
 #include <vector>
-#include <ctype.h>
 #include <dirent.h>
 #include <fcntl.h>
-#include <stddef.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <sys/stat.h>
 #include <unistd.h>
-
 									/*}}}*/
 namespace APT {
 // setDefaultConfigurationForCompressors				/*{{{*/
@@ -538,4 +538,29 @@ bool Configuration::isChroot()
    return once.res;
 }
 									/*}}}*/
+// isUsrMerged - whether usr is merged t			     	/*{{{*/
+// ---------------------------------------------------------------------
+/* */
+bool Configuration::checkUsrMerged()
+{
+   std::string rootDir = _config->FindDir("Dir");
+   for (auto dir : {"bin", "sbin", "lib"})
+   {
+      struct stat root;
+      struct stat usr;
+      std::string dirInRoot = rootDir + dir;
+      std::string dirInUsr = rootDir + "usr/" + dir;
+
+      // Missing directories are a boot strap scenario that needs to work
+      if (stat(dirInRoot.c_str(), &root))
+	 continue;
+      if (stat(dirInUsr.c_str(), &usr))
+	 continue;
+      if (root.st_ino != usr.st_ino)
+	 return _error->Warning("%s resolved to a different inode than %s", dirInRoot.c_str(), dirInUsr.c_str()), false;
+   }
+
+   return true;
+}
+									/*}}}*/
 }
