@@ -18,14 +18,14 @@
 #include <apt-pkg/hashes.h>
 #include <apt-pkg/strutl.h>
 
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <iostream>
 #include <string>
 #include <vector>
 #include <dirent.h>
 #include <fcntl.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <sys/stat.h>
 #include <sys/statvfs.h>
 #include <unistd.h>
@@ -71,7 +71,7 @@ bool IsMounted(string &Path)
 // UnmountCdrom - Unmount a cdrom					/*{{{*/
 // ---------------------------------------------------------------------
 /* Forking umount works much better than the umount syscall which can 
-   leave /etc/mtab inconsitant. We drop all messages this produces. */
+   leave /etc/mtab inconsistent. We drop all messages this produces. */
 bool UnmountCdrom(string Path)
 {
    // do not generate errors, even if the mountpoint does not exist
