@@ -26,12 +26,12 @@
 
 #include <algorithm>
 #include <array>
+#include <csignal>
+#include <cstring>
 #include <iostream>
 #include <memory>
 #include <string>
 #include <fcntl.h>
-#include <signal.h>
-#include <string.h>
 #include <unistd.h>
 
 #include <apti18n.h>
