@@ -17,6 +17,7 @@
 #include <apt-pkg/tagfile.h>
 
 #include <algorithm>
+#include <cassert>
 #include <map>
 #include <optional>
 #include <sstream>
@@ -24,7 +25,7 @@
 #include <utility>
 #include <vector>
 
-#include <string.h>
+#include <cstring>
 #include <sys/stat.h>
 
 #include <apti18n.h>
@@ -123,6 +124,9 @@ class APT_HIDDEN debReleaseIndexPrivate					/*{{{*/
    time_t DateMaxFuture;
    time_t NotBefore;
 
+   std::string Snapshot;
+   std::string SnapshotsServer;
+
    std::vector<std::string> Architectures;
    std::vector<std::string> NoSupportForAll;
    std::vector<std::string> SupportedComponents;
@@ -479,6 +483,7 @@ bool debReleaseIndex::Load(std::string const &Filename, std::string * const Erro
    Suite = Section.FindS("Suite");
    Codename = Section.FindS("Codename");
    ReleaseNotes = Section.FindS("Release-Notes");
+   d->SnapshotsServer = Section.FindS("Snapshots");
    {
       std::string const archs = Section.FindS("Architectures");
       if (archs.empty() == false)
@@ -789,6 +794,18 @@ bool debReleaseIndex::SetDateMaxFuture(time_t const DateMaxFuture)
       return _error->Error(_("Conflicting values set for option %s regarding source %s %s"), "Date-Max-Future", URI.c_str(), Dist.c_str());
    return true;
 }
+bool debReleaseIndex::SetSnapshot(std::string const Snapshot)
+{
+   if (d->Snapshot.empty())
+      d->Snapshot = Snapshot;
+   else if (d->Snapshot != Snapshot)
+      return _error->Error(_("Conflicting values set for option %s regarding source %s %s"), "Snapshot", URI.c_str(), Dist.c_str());
+   return true;
+}
+std::string debReleaseIndex::GetSnapshotsServer() const
+{
+   return d->SnapshotsServer;
+}
 bool debReleaseIndex::SetSignedBy(std::string const &pSignedBy)
 {
    if (SignedBy.empty() == true && pSignedBy.empty() == false)
@@ -1123,6 +1140,17 @@ class APT_HIDDEN debSLTypeDebian : public pkgSourceList::Type		/*{{{*/
 	 return defVal;
       return StringToBool(opt->second, defVal);
    }
+   static std::string GetSnapshotOption(std::map<std::string, std::string> const &Options, char const * const name, const std::string defVal="")
+   {
+      std::map<std::string, std::string>::const_iterator const opt = Options.find(name);
+      if (opt == Options.end())
+	 return defVal;
+      int boolVal = StringToBool(opt->second, -1);
+      if (boolVal != -1)
+	 return boolVal ? _config->Find("APT::Snapshot") : "";
+      return opt->second;
+   }
+
 
    static std::vector<std::string> GetMapKeys(std::map<std::string, std::string> const &Options)
    {
@@ -1131,6 +1159,8 @@ class APT_HIDDEN debSLTypeDebian : public pkgSourceList::Type		/*{{{*/
       std::transform(Options.begin(), Options.end(), std::back_inserter(ret),
 		     [](auto &&O) { return O.first; });
       std::sort(ret.begin(), ret.end());
+      auto r = std::remove(ret.begin(), ret.end(), "SHADOWED");
+      ret.erase(r, ret.end());
       return ret;
    }
 
@@ -1167,7 +1197,7 @@ class APT_HIDDEN debSLTypeDebian : public pkgSourceList::Type		/*{{{*/
       return true;
    }
 
-   static debReleaseIndex * GetDebReleaseIndexBy(std::vector<metaIndex *> &List, std::string const &URI,
+   static debReleaseIndex * GetDebReleaseIndexBy(std::vector<metaIndex *> &List, std::string URI,
 			   std::string const &Dist, std::map<std::string, std::string> const &Options)
    {
       std::map<std::string, std::string> ReleaseOptions{{
@@ -1182,6 +1212,8 @@ class APT_HIDDEN debSLTypeDebian : public pkgSourceList::Type		/*{{{*/
 	 ReleaseOptions.emplace("ALLOW_WEAK", "true");
       if (GetBoolOption(Options, "allow-downgrade-to-insecure", _config->FindB("Acquire::AllowDowngradeToInsecureRepositories")))
 	 ReleaseOptions.emplace("ALLOW_DOWNGRADE_TO_INSECURE", "true");
+      if (GetBoolOption(Options, "SHADOWED", false))
+	 ReleaseOptions.emplace("SHADOWED", "true");
 
       auto InReleasePath = Options.find("inrelease-path");
       if (InReleasePath != Options.end())
@@ -1221,10 +1253,132 @@ class APT_HIDDEN debSLTypeDebian : public pkgSourceList::Type		/*{{{*/
    }
 
    protected:
+   // This is a duplicate of pkgAcqChangelog::URITemplate()  with some changes to work 
+   // on metaIndex instead of cache structures, and using Snapshots
+   std::string SnapshotServer(debReleaseIndex const *Rls) const
+   {
+      if (Rls->GetLabel().empty() && Rls->GetOrigin().empty())
+	 return "";
+      std::string const serverConfig = "Acquire::Snapshots::URI";
+      std::string server;
+#define APT_EMPTY_SERVER        \
+   if (server.empty() == false) \
+   {                            \
+      return server;         \
+   }
+#define APT_CHECK_SERVER(X, Y)                                                                     \
+   if (not Rls->Get##X().empty())                                                                  \
+   {                                                                                               \
+      std::string const specialServerConfig = serverConfig + "::" + Y + #X + "::" + Rls->Get##X(); \
+      server = _config->Find(specialServerConfig);                                                 \
+      APT_EMPTY_SERVER                                                                             \
+   }
+      // this way e.g. Debian-Security can fallback to Debian
+      APT_CHECK_SERVER(Label, "Override::")
+      APT_CHECK_SERVER(Origin, "Override::")
 
-   bool CreateItemInternal(std::vector<metaIndex *> &List, std::string const &URI,
+      server = Rls->GetSnapshotsServer();
+      APT_EMPTY_SERVER
+
+      APT_CHECK_SERVER(Label, "")
+      APT_CHECK_SERVER(Origin, "")
+#undef APT_CHECK_SERVER
+#undef APT_EMPTY_SERVER
+      return "";
+   }
+
+   /// \brief Given a hostname, strip one level down, e.g. a.b.c -> .b.c -> .c, this
+   ///        allows you to match a.b.c against itself, .b.c, and .c, but not b.c
+   static inline std::string NextLevelDomain(std::string Host)
+   {
+      auto nextDot = Host.find(".", 1);
+      if (nextDot == Host.npos)
+	 return "";
+      return Host.substr(nextDot);
+   }
+   bool CreateItemInternal(std::vector<metaIndex *> &List, std::string URI,
 			   std::string const &Dist, std::string const &Section,
-			   bool const &IsSrc, std::map<std::string, std::string> const &Options) const
+			   bool const &IsSrc, std::map<std::string, std::string> Options) const
+   {
+      std::string SnapshotAptConf = _config->Find("APT::Snapshot");
+      std::string Snapshot = GetSnapshotOption(Options, "snapshot", SnapshotAptConf.empty() ? "" : SnapshotAptConf + "?");
+      if (not Snapshot.empty()) {
+	 std::map<std::string, std::string> SnapshotOptions = Options;
+
+	 Options.emplace("SHADOWED", "true");
+
+	 ::URI ArchiveURI(URI);
+	 // Trim trailing and leading / from the path because we don't want them when calculating snapshot url
+	 if (not ArchiveURI.Path.empty() && ArchiveURI.Path[ArchiveURI.Path.length() - 1] == '/')
+	    ArchiveURI.Path.erase(ArchiveURI.Path.length() - 1);
+	 if (not ArchiveURI.Path.empty() && ArchiveURI.Path[0] == '/')
+	    ArchiveURI.Path.erase(0, 1);
+	 std::string Server;
+
+	 auto const PreviousDeb = List.empty() ? nullptr : List.back();
+	 auto const Deb = GetDebReleaseIndexBy(List, URI, Dist, Options);
+	 std::string filename;
+
+	 // The Release file and config based on that should be the ultimate source of truth.
+	 if (Deb && ReleaseFileName(Deb, filename))
+	 {
+	    auto OldDeb = dynamic_cast<debReleaseIndex *>(Deb->UnloadedClone());
+	    if (not OldDeb->Load(filename, nullptr))
+	       return _error->Error("Cannot identify snapshot server for %s %s - run update without snapshot id first", URI.c_str(), Dist.c_str());
+	    Server = SnapshotServer(OldDeb);
+	    delete OldDeb;
+	 }
+	 // We did not find a server based on the release file.
+	 // Lookup a fallback based on the host. For a.b.c, this will
+	 // try a.b.c, .b.c, and .c to allow generalization for cc.archive.ubuntu.com
+	 if (Server.empty())
+	 {
+	    for (std::string Host = ArchiveURI.Host; not Host.empty(); Host = NextLevelDomain(Host))
+	    {
+	       Server = _config->Find("Acquire::Snapshots::URI::Host::" + Host);
+	       if (not Server.empty())
+		  break;
+	    }
+	 }
+	 if (Server.empty() || Server == "no")
+	 {
+	    if (APT::String::Endswith(Snapshot, "?"))
+	    {
+	       // Erase the SHADOWED option and remove the release index from the list if we created it.
+	       Options.erase("SHADOWED");
+	       if (Deb && Deb != PreviousDeb) {
+		  assert(List.back() == Deb);
+		  List.pop_back();
+		  delete Deb;
+	       }
+	       goto nosnapshot;
+	    }
+	    if (Server != "no" && filename.empty())
+	       return _error->Error("Cannot identify snapshot server for %s %s - run update without snapshot id first", URI.c_str(), Dist.c_str());
+	    return _error->Error("Snapshots not supported for %s %s", URI.c_str(), Dist.c_str());
+	 }
+	 // We have found a server by now, so we enable snapshots for this source.
+	 if (APT::String::Endswith(Snapshot, "?"))
+	 {
+	    Snapshot.pop_back();
+	 }
+
+	 assert(not Snapshot.empty());
+	 auto SnapshotURI = SubstVar(SubstVar(Server, "@SNAPSHOTID@", Snapshot), "@PATH@", ArchiveURI.Path);
+
+	 if (not CreateItemInternalOne(List, SnapshotURI, Dist, Section, IsSrc, SnapshotOptions))
+	    return false;
+      }
+   nosnapshot:
+      if (not CreateItemInternalOne(List, URI, Dist, Section, IsSrc, Options))
+	 return false;
+
+
+      return true;
+   }
+   bool CreateItemInternalOne(std::vector<metaIndex *> &List, std::string URI,
+			   std::string const &Dist, std::string const &Section,
+			   bool const &IsSrc, std::map<std::string, std::string> Options) const
    {
       auto const Deb = GetDebReleaseIndexBy(List, URI, Dist, Options);
       if (Deb == nullptr)
@@ -1263,9 +1417,13 @@ class APT_HIDDEN debSLTypeDebian : public pkgSourceList::Type		/*{{{*/
 	  Deb->SetValidUntilMax(GetTimeOption(Options, "valid-until-max")) == false ||
 	  Deb->SetValidUntilMin(GetTimeOption(Options, "valid-until-min")) == false ||
 	  Deb->SetCheckDate(GetTriStateOption(Options, "check-date")) == false ||
-	  Deb->SetDateMaxFuture(GetTimeOption(Options, "date-max-future")) == false)
+	  Deb->SetDateMaxFuture(GetTimeOption(Options, "date-max-future")) == false ||
+	  Deb->SetSnapshot(GetSnapshotOption(Options, "snapshot")) == false)
 	 return false;
 
+      if (GetBoolOption(Options, "sourceslist-entry-is-deb822", false))
+	 Deb->SetFlag(metaIndex::Flag::DEB822);
+
       std::map<std::string, std::string>::const_iterator const signedby = Options.find("signed-by");
       if (signedby == Options.end())
       {
