@@ -13,11 +13,10 @@
 #include <apt-pkg/packagemanager.h>
 #include <apt-pkg/pkgcache.h>
 
+#include <cstdio>
 #include <map>
 #include <string>
 #include <vector>
-#include <stdio.h>
-
 
 class pkgDepCache;
 namespace APT { namespace Progress { class PackageManager; } }
