@@ -29,18 +29,19 @@
 #include <apt-pkg/versionmatch.h>
 
 #include <algorithm>
+#include <cstdio>
+#include <cstring>
 #include <iostream>
-#include <memory>
-#include <sstream>
 #include <iterator>
 #include <list>
+#include <memory>
+#include <random>
 #include <set>
+#include <sstream>
 #include <string>
 #include <unordered_map>
 #include <utility>
 #include <vector>
-#include <stdio.h>
-#include <string.h>
 
 #include <sys/stat.h>
 
@@ -67,23 +68,37 @@ class DefaultRootSetFunc2 : public pkgDepCache::DefaultRootSetFunc
 
 									/*}}}*/
 // helper for Install-Recommends-Sections and Never-MarkAuto-Sections	/*{{{*/
-static bool 
-ConfigValueInSubTree(const char* SubTree, const char *needle)
+// FIXME: Has verbatim copy in cmdline/apt-mark.cc
+static bool ConfigValueInSubTree(const char* SubTree, std::string_view const needle)
 {
-   Configuration::Item const *Opts;
-   Opts = _config->Tree(SubTree);
-   if (Opts != 0 && Opts->Child != 0)
+   if (needle.empty())
+      return false;
+   Configuration::Item const *Opts = _config->Tree(SubTree);
+   if (Opts != nullptr && Opts->Child != nullptr)
    {
       Opts = Opts->Child;
-      for (; Opts != 0; Opts = Opts->Next)
+      for (; Opts != nullptr; Opts = Opts->Next)
       {
-	 if (Opts->Value.empty() == true)
+	 if (Opts->Value.empty())
 	    continue;
-	 if (strcmp(needle, Opts->Value.c_str()) == 0)
+	 if (needle == Opts->Value)
 	    return true;
       }
    }
    return false;
+}
+static bool SectionInSubTree(char const * const SubTree, std::string_view Needle)
+{
+   if (ConfigValueInSubTree(SubTree, Needle))
+      return true;
+   auto const sub = Needle.rfind('/');
+   if (sub == std::string_view::npos)
+   {
+      std::string special{"/"};
+      special.append(Needle);
+      return ConfigValueInSubTree(SubTree, special);
+   }
+   return ConfigValueInSubTree(SubTree, Needle.substr(sub + 1));
 }
 									/*}}}*/
 pkgDepCache::ActionGroup::ActionGroup(pkgDepCache &cache) :		/*{{{*/
@@ -127,6 +142,7 @@ struct pkgDepCache::Private
 {
    std::unique_ptr<InRootSetFunc> inRootSetFunc;
    std::unique_ptr<APT::CacheFilter::Matcher> IsAVersionedKernelPackage, IsProtectedKernelPackage;
+   std::string machineID;
 };
 pkgDepCache::pkgDepCache(pkgCache *const pCache, Policy *const Plcy) : group_level(0), Cache(pCache), PkgState(0), DepState(0),
 								       iUsrSize(0), iDownloadSize(0), iInstCount(0), iDelCount(0), iKeepCount(0),
@@ -134,6 +150,7 @@ pkgDepCache::pkgDepCache(pkgCache *const pCache, Policy *const Plcy) : group_lev
 {
    DebugMarker = _config->FindB("Debug::pkgDepCache::Marker", false);
    DebugAutoInstall = _config->FindB("Debug::pkgDepCache::AutoInstall", false);
+   d->machineID = APT::Configuration::getMachineID();
    delLocalPolicy = 0;
    LocalPolicy = Plcy;
    if (LocalPolicy == 0)
@@ -1050,7 +1067,7 @@ bool pkgDepCache::MarkDelete(PkgIterator const &Pkg, bool rPurge,
 	 // We do not check for or-groups here as we don't know which package takes care of
 	 // providing the feature the user likes e.g.:  browser1 | browser2 | browser3
 	 // Temporary removals are effected by this as well, which is bad, but unlikely in practice
-	 bool const PinNeverMarkAutoSection = (PV->Section != 0 && ConfigValueInSubTree("APT::Never-MarkAuto-Sections", PV.Section()));
+	 bool const PinNeverMarkAutoSection = (PV->Section != 0 && SectionInSubTree("APT::Never-MarkAuto-Sections", PV.Section()));
 	 if (PinNeverMarkAutoSection)
 	 {
 	    for (DepIterator D = PV.DependsList(); D.end() != true; ++D)
@@ -1761,8 +1778,8 @@ bool pkgDepCache::MarkInstall(PkgIterator const &Pkg, bool AutoInst,
       VerIterator const CurVer = Pkg.CurrentVer();
       if (not CurVer.end() && CurVer->Section != 0 && strcmp(CurVer.Section(), PV.Section()) != 0)
       {
-	 bool const CurVerInMoveSection = ConfigValueInSubTree("APT::Move-Autobit-Sections", CurVer.Section());
-	 bool const InstVerInMoveSection = ConfigValueInSubTree("APT::Move-Autobit-Sections", PV.Section());
+	 bool const CurVerInMoveSection = SectionInSubTree("APT::Move-Autobit-Sections", CurVer.Section());
+	 bool const InstVerInMoveSection = SectionInSubTree("APT::Move-Autobit-Sections", PV.Section());
 	 return (not CurVerInMoveSection && InstVerInMoveSection);
       }
       return false;
@@ -2254,7 +2271,7 @@ bool pkgDepCache::Policy::IsImportantDep(DepIterator const &Dep) const
       // FIXME: this is a meant as a temporary solution until the
       //        recommends are cleaned up
       const char *sec = Dep.ParentVer().Section();
-      if (sec && ConfigValueInSubTree("APT::Install-Recommends-Sections", sec))
+      if (sec && SectionInSubTree("APT::Install-Recommends-Sections", sec))
 	 return true;
    }
    else if(Dep->Type == pkgCache::Dep::Suggests)
@@ -2548,3 +2565,51 @@ bool pkgDepCache::MarkAndSweep()
       return false;
 }
 									/*}}}*/
+
+// DepCache::PhasingApplied					/*{{{*/
+// Check if this version is a phased update that should be ignored, not considering whether
+// it is a security update.
+static bool IsIgnoredPhasedUpdate(std::string machineID, pkgCache::VerIterator const &Ver)
+{
+   if (_config->FindB("APT::Get::Phase-Policy", false))
+      return false;
+
+   // The order and fallbacks for the always/never checks come from update-manager and exist
+   // to preserve compatibility.
+   if (_config->FindB("APT::Get::Always-Include-Phased-Updates",
+		      _config->FindB("Update-Manager::Always-Include-Phased-Updates", false)))
+      return false;
+
+   if (_config->FindB("APT::Get::Never-Include-Phased-Updates",
+		      _config->FindB("Update-Manager::Never-Include-Phased-Updates", false)))
+      return true;
+
+   if (machineID.empty()			 // no machine-id
+       || getenv("SOURCE_DATE_EPOCH") != nullptr // reproducible build - always include
+       || APT::Configuration::isChroot())
+      return false;
+
+   std::string seedStr = std::string(Ver.SourcePkgName()) + "-" + Ver.SourceVerStr() + "-" + machineID;
+   std::seed_seq seed(seedStr.begin(), seedStr.end());
+   std::minstd_rand rand(seed);
+   std::uniform_int_distribution<unsigned int> dist(0, 100);
+
+   return dist(rand) > Ver.PhasedUpdatePercentage();
+}
+
+bool pkgDepCache::PhasingApplied(pkgCache::PkgIterator Pkg) const
+{
+   if (Pkg->CurrentVer == 0)
+      return false;
+   if ((*this)[Pkg].CandidateVer == 0)
+      return false;
+   if ((*this)[Pkg].CandidateVerIter(*Cache).PhasedUpdatePercentage() == 100)
+      return false;
+   if ((*this)[Pkg].CandidateVerIter(*Cache).IsSecurityUpdate())
+      return false;
+   if (!IsIgnoredPhasedUpdate(d->machineID, (*this)[Pkg].CandidateVerIter(*Cache)))
+      return false;
+
+   return true;
+}
+									/*}}}*/
