@@ -9,6 +9,7 @@
 // Include Files							/*{{{*/
 #include <config.h>
 
+#include <apt-pkg/acquire.h>
 #include <apt-pkg/aptconfiguration.h>
 #include <apt-pkg/configuration.h>
 #include <apt-pkg/deblistparser.h>
@@ -120,6 +121,7 @@ std::string IndexTarget::Option(OptionKeys const EnumKey) const		/*{{{*/
       APT_CASE(ALLOW_WEAK);
       APT_CASE(ALLOW_DOWNGRADE_TO_INSECURE);
       APT_CASE(INRELEASE_PATH);
+      APT_CASE(SHADOWED);
 #undef APT_CASE
       case FILENAME:
       {
@@ -174,7 +176,7 @@ pkgDebianIndexTargetFile::pkgDebianIndexTargetFile(IndexTarget const &Target, bo
 									/*}}}*/
 std::string pkgDebianIndexTargetFile::ArchiveURI(std::string const &File) const/*{{{*/
 {
-   return Target.Option(IndexTarget::REPO_URI) + File;
+   return Target.Option(IndexTarget::REPO_URI) + pkgAcquire::URIEncode(File);
 }
 									/*}}}*/
 std::string pkgDebianIndexTargetFile::Describe(bool const Short) const	/*{{{*/
@@ -220,6 +222,8 @@ unsigned long pkgDebianIndexTargetFile::Size() const				/*{{{*/
 									/*}}}*/
 bool pkgDebianIndexTargetFile::Exists() const					/*{{{*/
 {
+   if (Target.OptionBool(IndexTarget::SHADOWED))
+      return false;
    return FileExists(IndexFileName());
 }
 									/*}}}*/
@@ -281,7 +285,7 @@ std::string pkgDebianIndexRealFile::Describe(bool const /*Short*/) const/*{{{*/
 									/*}}}*/
 std::string pkgDebianIndexRealFile::ArchiveURI(std::string const &/*File*/) const/*{{{*/
 {
-   return "file:" + File;
+   return "file:" + pkgAcquire::URIEncode(File);
 }
 									/*}}}*/
 std::string pkgDebianIndexRealFile::IndexFileName() const			/*{{{*/
