@@ -66,7 +66,7 @@ static std::vector<aptDispatchWithHelp> GetCommands()			/*{{{*/
       {"install", &DoInstall, _("install packages")},
       {"reinstall", &DoInstall, _("reinstall packages")},
       {"remove", &DoInstall, _("remove packages")},
-      {"autoremove", &DoInstall, _("Remove automatically all unused packages")},
+      {"autoremove", &DoInstall, _("automatically remove all unused packages")},
       {"auto-remove", &DoInstall, nullptr},
       {"autopurge",&DoInstall, nullptr},
       {"purge", &DoInstall, nullptr},
@@ -89,6 +89,8 @@ static std::vector<aptDispatchWithHelp> GetCommands()			/*{{{*/
       {"policy",&Policy, nullptr},
       {"build-dep", &DoBuildDep,nullptr},
       {"clean", &DoClean, nullptr},
+      {"distclean", &DoDistClean, nullptr},
+      {"dist-clean", &DoDistClean, nullptr},
       {"autoclean", &DoAutoClean, nullptr},
       {"auto-clean", &DoAutoClean, nullptr},
       {"source", &DoSource, nullptr},
