@@ -19,16 +19,18 @@
 #include <apt-pkg/srvrec.h>
 #include <apt-pkg/strutl.h>
 
+#ifdef HAVE_GNUTLS
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
+#endif
 
+#include <cerrno>
+#include <cstdio>
+#include <cstring>
 #include <list>
 #include <set>
 #include <sstream>
 #include <string>
-#include <errno.h>
-#include <stdio.h>
-#include <string.h>
 #include <unistd.h>
 
 // Internet stuff
@@ -798,7 +800,8 @@ ResultState UnwrapSocks(std::string Host, int Port, URI Proxy, std::unique_ptr<M
 
    return ResultState::SUCCESSFUL;
 }
-									/*}}}*/
+
+#ifdef HAVE_GNUTLS									/*}}}*/
 // UnwrapTLS - Handle TLS connections 					/*{{{*/
 // ---------------------------------------------------------------------
 /* Performs a TLS handshake on the socket */
@@ -894,7 +897,8 @@ struct TlsFd : public MethodFd
 };
 
 ResultState UnwrapTLS(std::string const &Host, std::unique_ptr<MethodFd> &Fd,
-		      unsigned long Timeout, aptMethod *Owner)
+		      unsigned long const Timeout, aptMethod * const Owner,
+		      aptConfigWrapperForMethods const * const OwnerConf)
 {
    if (_config->FindB("Acquire::AllowTLS", true) == false)
    {
@@ -940,7 +944,7 @@ ResultState UnwrapTLS(std::string const &Host, std::unique_ptr<MethodFd> &Fd,
    }
 
    // Credential setup
-   std::string fileinfo = Owner->ConfigFind("CaInfo", "");
+   std::string fileinfo = OwnerConf->ConfigFind("CaInfo", "");
    if (fileinfo.empty())
    {
       // No CaInfo specified, use system trust store.
@@ -965,20 +969,20 @@ ResultState UnwrapTLS(std::string const &Host, std::unique_ptr<MethodFd> &Fd,
       }
    }
 
-   if (!Owner->ConfigFind("IssuerCert", "").empty())
+   if (not OwnerConf->ConfigFind("IssuerCert", "").empty())
    {
       _error->Error("The option '%s' is not supported anymore", "IssuerCert");
       return ResultState::FATAL_ERROR;
    }
-   if (!Owner->ConfigFind("SslForceVersion", "").empty())
+   if (not OwnerConf->ConfigFind("SslForceVersion", "").empty())
    {
       _error->Error("The option '%s' is not supported anymore", "SslForceVersion");
       return ResultState::FATAL_ERROR;
    }
 
    // For client authentication, certificate file ...
-   std::string const cert = Owner->ConfigFind("SslCert", "");
-   std::string const key = Owner->ConfigFind("SslKey", "");
+   std::string const cert = OwnerConf->ConfigFind("SslCert", "");
+   std::string const key = OwnerConf->ConfigFind("SslKey", "");
    if (cert.empty() == false)
    {
       if ((err = gnutls_certificate_set_x509_key_file(
@@ -993,7 +997,7 @@ ResultState UnwrapTLS(std::string const &Host, std::unique_ptr<MethodFd> &Fd,
    }
 
    // CRL file
-   std::string const crlfile = Owner->ConfigFind("CrlFile", "");
+   std::string const crlfile = OwnerConf->ConfigFind("CrlFile", "");
    if (crlfile.empty() == false)
    {
       if ((err = gnutls_certificate_set_x509_crl_file(tlsFd->credentials,
@@ -1017,9 +1021,9 @@ ResultState UnwrapTLS(std::string const &Host, std::unique_ptr<MethodFd> &Fd,
       return ResultState::FATAL_ERROR;
    }
 
-   if (Owner->ConfigFindB("Verify-Peer", true))
+   if (OwnerConf->ConfigFindB("Verify-Peer", true))
    {
-      gnutls_session_set_verify_cert(tlsFd->session, Owner->ConfigFindB("Verify-Host", true) ? tlsFd->hostname.c_str() : nullptr, 0);
+      gnutls_session_set_verify_cert(tlsFd->session, OwnerConf->ConfigFindB("Verify-Host", true) ? tlsFd->hostname.c_str() : nullptr, 0);
    }
 
    // set SNI only if the hostname is really a name and not an address
@@ -1049,4 +1053,4 @@ ResultState UnwrapTLS(std::string const &Host, std::unique_ptr<MethodFd> &Fd,
 
    return ResultState::SUCCESSFUL;
 }
-									/*}}}*/
+#endif									/*}}}*/
